#include <libplayerc++/playerc++.h>
#include <math.h>

int main(int argc, char *argv[]) {
	using namespace PlayerCc;
	// set up proxies
	PlayerClient robot("localhost", 6665);
	Position2dProxy pp(&robot, 0);
	SimulationProxy simp(&robot, 0);
	// declare constants
	const double KPO = 1.0;
	const double KPI = 0.6;
	const int NUMBER_OF_POINTS = 6;
	const int POINTS[NUMBER_OF_POINTS][2] = {{-7, -5}, {5, -5}, {6, 6}, {-6, 0}, {7, -7}, {9, -7}};
	const double PI = 3.14159265;
	const double SPEED = 0.2;
	// declare variables
	int i = 0, j = 0;
	double error, constant, closestPointX, closestPointY, distanceToPoint, totalError = 0, thetaC, thetaL, thetaR, turnRate;
	std::cout << "KPO = " <<  KPO << " KPI = " << KPI << std::endl;
	// set intial position
  	simp.SetPose2d (const_cast<char*>("robot1"), -7, -4.7, 0);
	for(;;) {
		robot.Read();
		// calculate closest point on line
		constant = ((pp.GetXPos()-POINTS[i][0])*(POINTS[i+1][0]-POINTS[i][0])+(pp.GetYPos()-POINTS[i][1])*(POINTS[i+1][1]-POINTS[i][1]))/(pow((POINTS[i+1][0]-POINTS[i][0]), 2)+pow((POINTS[i+1][1]-POINTS[i][1]), 2)); 
		closestPointX = POINTS[i][0]+constant*(POINTS[i+1][0]-POINTS[i][0]);
		closestPointY = POINTS[i][1]+constant*(POINTS[i+1][1]-POINTS[i][1]);
		// calculate error
		error = sqrt(pow(closestPointX-pp.GetXPos(), 2)+pow(closestPointY-pp.GetYPos(), 2));
		totalError += error;
		if ((pp.GetXPos()-closestPointX+pp.GetYPos()-closestPointY) < 0) { error *= -1; }
		// calculate distance to last point of the line		
		distanceToPoint = sqrt(pow((pp.GetXPos()-POINTS[i+1][0]), 2) + pow((pp.GetYPos()-POINTS[i+1][1]), 2));
		if (distanceToPoint < 0.5) { i++; }
		// if last point reached then stop
		if (i == (NUMBER_OF_POINTS-1)) { break; }
		// calculate angle of line
		if ((POINTS[i+1][0]-POINTS[i][0]) == 0) {
			thetaL = 0;
		} else {
			thetaL = atan2(POINTS[i+1][1]-POINTS[i][1], POINTS[i+1][0]-POINTS[i][0]);
		}
		// cascading loops
		thetaC = -KPO * error;
		thetaR = NORMALIZE(pp.GetYaw()) - thetaL;
		if (thetaC > PI/2)
		   { thetaC = PI/2; }
		if (thetaC < -PI/2)
		   { thetaC = -PI/2; }
		 
		turnRate = KPI * (thetaC - thetaR);
		// printing position every time adds too much delay...
		if (fmod(j, 5) == 0) {
			double x, y, a;
		   	simp.GetPose2d(const_cast<char*>("robot1"), x, y, a);
			std::cout << x << ", " << y << ", " << turnRate << ", " << error << " thetaC " << thetaC << std::endl;
			printf("DX=%.2f   DY=%.2f  errorgain:%d\n", pp.GetXPos() - closestPointX, pp.GetYPos()-closestPointY,((pp.GetXPos()-closestPointX+pp.GetYPos()-closestPointY) < 0) );
		}
		pp.SetSpeed(SPEED, turnRate);
		j++;
	}
	std::cout << "Average error: " <<  totalError/j << std::endl;
}
